/*
 * Hangman application
 * BME MIT, IRF course
 * https://www.inf.mit.bme.hu/edu/bsc/irf
 */
package hangman;

import java.util.Set;
import java.util.TreeSet;

/**
 * Simple class that server as the main data type for the hangman game.
 *
 * @author Tamas Toth
 */
public class Puzzle {

    public final static String specialChars = " -_.:?!,'";
    private String category;
    private String riddle;
    //set of special characters and letters already asked 
    private Set<Character> asked;

    /**
     *
     * @param riddle The riddle to guess
     * @param category Category (e.g. film, novel, etc...) of the riddle
     */
    public Puzzle(String riddle, String category) {

        this.riddle = riddle;
        this.category = category;

        asked = new TreeSet<>();

        //all special characters should be treated as asked letters
        for (char letter : specialChars.toCharArray()) {
            asked.add(letter);
        }
    }

    /**
     * Ask a single letter
     *
     * @param letter case-insensitive letter to be asked.
     */
    public void Ask(char letter) {
        asked.add(new Character(letter).toString().toLowerCase().charAt(0));
        asked.add(new Character(letter).toString().toUpperCase().charAt(0));
    }

    /**
     * Decide whether a letter contained in the riddle had been asked.
     *
     * @param letter The letter in question.
     * @return Is the letter revealed?
     */
    public boolean isRevealed(char letter) {
        return (isContained(letter) && isAsked(letter));
    }

    /**
     * Decide whether a case-insensitive letter is contained in the riddle.
     *
     * @param letter The letter in question.
     * @return Is the letter contained in the riddle?
     */
    public boolean isContained(char letter) {
        return (riddle.toLowerCase().indexOf(letter) >= 0 || riddle.toUpperCase().indexOf(letter) >= 0);
    }

    /**
     * Decide whether a letter has been asked (or is a special character).
     *
     * @param letter The letter in question.
     * @return Has the character been asked?
     */
    public boolean isAsked(char letter) {
        return asked.contains(new Character(letter));
    }

    /**
     * Decide whether the riddle has been solved, that means, all contained
     * characters have been revealed.
     *
     * @return Has the riddle been solved?
     */
    public boolean isSolved() {
        for (char letter : riddle.toCharArray()) {
            if (!asked.contains(letter)) {
                return false;
            }
        }
        return true;
    }

    /**
     * Decide whether a string solves the riddle.
     *
     * @param guess The guess for the riddle.
     * @return Does the guess solve the riddle?
     */
    public boolean isSolution(String guess) {
        return guess.equalsIgnoreCase(new String(riddle));
    }

    /**
     * Try to solve the riddle; if the guess solves the riddle, then all
     * characters will be revealed.
     *
     * @param guess The guess for the riddle.
     *
     */
    public void Guess(String guess) {
        if (isSolution(guess)) {
            for (char letter : riddle.toCharArray()) {
                asked.add(letter);
            }
        }
    }

    /**
     * The category of the riddle.
     *
     * @return The category of the riddle.
     */
    public String getCategory() {
        return category;
    }

    @Override
    public String toString() {
        char[] result = riddle.toCharArray();

        //if a character in the riddle had not been asked, then it should be masked with a *
        for (int i = 0; i < result.length; i++) {
            if (!isAsked(result[i])) {
                result[i] = '*';
            }
        }
        return new String(result);
    }
}